@@ -1,7 +1,7 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 #    test-postfix.py quality assurance test script for postfix
-#    Copyright (C) 2008-2012 Canonical Ltd.
+#    Copyright (C) 2008-2021 Canonical Ltd.
 #    Author: Kees Cook <kees@ubuntu.com>
 #    Author: Marc Deslauriers <marc.deslauriers@canonical.com>
 #    Author: Jamie Strandboge <jamie@canonical.com>
@@ -18,7 +18,7 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-# QRT-Packages: postfix sasl2-bin procmail python-pexpect
+# QRT-Packages: postfix sasl2-bin procmail python3-pexpect
 # QRT-Privilege: root
 # QRT-Conflicts: exim4
 
@@ -27,7 +27,7 @@
     not work if "Local Only" was selected.
 
     How to run against a clean schroot named 'hardy':
-        schroot -c hardy -u root -- sh -c 'apt-get -y install procmail python-unit postfix sasl2-bin python-pexpect lsb-release ssl-cert && ./test-postfix.py -v'
+        schroot -c hardy -u root -- sh -c 'apt-get -y install procmail postfix sasl2-bin python-pexpect lsb-release && ./test-postfix.py -v'
 
     Tests:
       00: setup
@@ -38,11 +38,9 @@
       99: restore configs
 '''
 
-import unittest, subprocess, re, pexpect, smtplib, socket, os, time, tempfile
+import unittest, subprocess, pexpect, smtplib, socket, os, time
 import testlib
 
-import sys
-
 class PostfixTest(testlib.TestlibCase):
     '''Test Postfix MTA.'''
 
@@ -62,8 +60,8 @@ class PostfixTest(testlib.TestlibCase):
 
         conf_file = '/etc/postfix/main.cf'
         # Use mbox only
-        testlib.config_comment(conf_file,'home_mailbox')
-        testlib.config_set(conf_file,'mailbox_command','procmail -a "$EXTENSION"')
+        testlib.config_comment(conf_file, 'home_mailbox')
+        testlib.config_set(conf_file, 'mailbox_command', 'procmail -a "$EXTENSION"')
 
         # Turn on sasl
         self._setup_sasl("PLAIN")
@@ -90,15 +88,15 @@ class PostfixTest(testlib.TestlibCase):
             testlib.config_restore(f)
 
         # put saslauthd back
-        for f in ['/var/spool/postfix/var/run/saslauthd', '/var/run/saslauthd']:
-            if os.path.isfile(f) or os.path.islink(f):
-                os.unlink(f)
-            elif os.path.exists(f):
-                testlib.recursive_rm(f)
-        subprocess.call(['mkdir','-p','/var/run/saslauthd'])
-        subprocess.call(['/usr/sbin/service', 'saslauthd', 'stop'], stdout=subprocess.PIPE)
-        subprocess.call(['/usr/sbin/service', 'saslauthd', 'start'], stdout=subprocess.PIPE)
-        subprocess.call(['cp', '/etc/sasldb2', '/var/spool/postfix/etc/sasldb2'])
+        if self.lsb_release['Release'] < 25.10:
+            for f in ['/var/spool/postfix/var/run/saslauthd', '/var/run/saslauthd']:
+                if os.path.isfile(f) or os.path.islink(f):
+                    os.unlink(f)
+                elif os.path.exists(f):
+                    testlib.recursive_rm(f)
+            subprocess.call(['mkdir', '-p', '/var/run/saslauthd'])
+        subprocess.call(['/etc/init.d/saslauthd', 'stop'], stdout=subprocess.PIPE)
+        subprocess.call(['/etc/init.d/saslauthd', 'start'], stdout=subprocess.PIPE)
 
     def tearDown(self):
         '''Clean up after each test_* function'''
@@ -111,21 +109,21 @@ class PostfixTest(testlib.TestlibCase):
 
     def _restart_server(self):
         '''Restart server'''
-        subprocess.call(['/usr/sbin/service', 'postfix', 'stop'], stdout=subprocess.PIPE)
-        assert subprocess.call(['/usr/sbin/service', 'postfix', 'start'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT) == 0
+        subprocess.call(['/etc/init.d/postfix', 'stop'], stdout=subprocess.PIPE)
+        assert subprocess.call(['/etc/init.d/postfix', 'start'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT) == 0
         # Postfix exits its init script before the master listener has started
         time.sleep(2)
 
     def _setup_sasl(self, mech, other_mech="", force_sasldb=False):
         '''Setup sasl for mech'''
         conf_file = '/etc/postfix/main.cf'
-        for field in ['smtpd_sasl_type','smtpd_sasl_local_domain','smtpd_tls_auth_only']:
-            testlib.config_comment(conf_file,field)
-        testlib.config_set(conf_file,'smtpd_sasl_path','smtpd')
-        testlib.config_set(conf_file,'smtpd_sasl_auth_enable','yes')
+        for field in ['smtpd_sasl_type', 'smtpd_sasl_local_domain', 'smtpd_tls_auth_only']:
+            testlib.config_comment(conf_file, field)
+        testlib.config_set(conf_file, 'smtpd_sasl_path', 'smtpd')
+        testlib.config_set(conf_file, 'smtpd_sasl_auth_enable', 'yes')
         #testlib.config_set(conf_file,'broken_sasl_auth_clients','yes')
-        testlib.config_set(conf_file,'smtpd_sasl_authenticated_header','yes')
-        testlib.config_set(conf_file,'smtpd_tls_loglevel','2')
+        testlib.config_set(conf_file, 'smtpd_sasl_authenticated_header', 'yes')
+        testlib.config_set(conf_file, 'smtpd_tls_loglevel', '2')
 
         # setup smtpd.conf and the sasl users
         contents = ''
@@ -143,15 +141,16 @@ allowplaintext: 1
 mech_list: %s %s
 ''' % (mech, other_mech)
 
-            # attach SASL to postfix chroot
-            subprocess.call(['mkdir','-p','/var/spool/postfix/var/run/saslauthd'])
-            subprocess.call(['rm','-rf','/var/run/saslauthd'])
-            subprocess.call(['ln','-s','/var/run/saslauthd', '/var/spool/postfix/var/run/saslauthd'])
-            subprocess.call(['/usr/sbin/service', 'saslauthd', 'stop'], stdout=subprocess.PIPE)
-            assert subprocess.call(['/usr/sbin/service', 'saslauthd', 'start'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT) == 0
+            # attach SASL to postfix chroot (not needed in questing+)
+            if self.lsb_release['Release'] < 25.10:
+                subprocess.call(['mkdir', '-p', '/var/spool/postfix/var/run/saslauthd'])
+                subprocess.call(['rm', '-rf', '/var/run/saslauthd'])
+                subprocess.call(['ln', '-s', '/var/spool/postfix/var/run/saslauthd', '/var/run/saslauthd'])
+            subprocess.call(['/etc/init.d/saslauthd', 'stop'], stdout=subprocess.PIPE)
+            assert subprocess.call(['/etc/init.d/saslauthd', 'start'], stdout=subprocess.PIPE, stderr=subprocess.STDOUT) == 0
 
             # Force crackful perms so chroot'd postfix can talk to saslauthd
-            subprocess.call(['chmod','o+x','/var/run/saslauthd'])
+            #subprocess.call(['chmod', 'o+x', '/var/spool/postfix/var/run/saslauthd'])
         else:
             plaintext = "1"
             if mech == "LOGIN" or mech == "PLAIN":
@@ -173,7 +172,7 @@ mech_list: %s %s
 
             child = pexpect.spawn('saslpasswd2 -c -u %s %s' % (report.strip(), self.user.login))
             time.sleep(0.2)
-            child.expect(r'(?i)password', timeout=5)
+            child.expect(r'.*[pP]assword', timeout=5)
             time.sleep(0.2)
             child.sendline(self.user.password)
             time.sleep(0.2)
@@ -207,13 +206,13 @@ mech_list: %s %s
         '''Is the server listening'''
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         s.settimeout(5)
-        s.connect(('localhost',2525))
+        s.connect(('localhost', 2525))
         greeting = s.recv(1024).decode('utf-8')
         # 220 gorgon.outflux.net ESMTP Postfix (Ubuntu)
-        self.assertTrue(greeting.startswith('220 '),greeting)
-        self.assertTrue('ESMTP' in greeting,greeting)
-        self.assertTrue('Postfix' in greeting,greeting)
-        self.assertFalse('MTA' in greeting,greeting)
+        self.assertTrue(greeting.startswith('220 '), greeting)
+        self.assertTrue('ESMTP' in greeting, greeting)
+        self.assertTrue('Postfix' in greeting, greeting)
+        self.assertFalse('MTA' in greeting, greeting)
         s.close()
 
     def test_00_listening(self):
@@ -224,7 +223,7 @@ mech_list: %s %s
         self._is_listening()
 
     def _vrfy(self, address, valid = True):
-        self.s.putcmd("vrfy",address)
+        self.s.putcmd("vrfy", address)
         code, msg = self.s.getreply()
         reply = '%d %s' % (code, msg)
         if valid:
@@ -245,12 +244,22 @@ mech_list: %s %s
         self.assertEqual(code, 250, reply)
         self.assertEqual(self.s.does_esmtp, 1, reply)
         self.assertTrue(b'8BITMIME' in self.s.ehlo_resp, reply)
-        # Help available as of postfix 3.9
+        # No help available
         self.s.putcmd("help")
         code, msg = self.s.getreply()
         reply = '%d %s' % (code, msg)
-        self.assertEqual(code, 214, reply)
-        self.assertTrue(b'2.0.0 Commands:' in msg, reply)
+
+        if self.lsb_release['Release'] < 24.04:
+            expected_code = 502
+        elif self.lsb_release['Release'] >= 25.10:
+            expected_code = 214
+        else:
+            expected_code = 500
+
+        self.assertEqual(code, expected_code, reply)
+
+        if self.lsb_release['Release'] < 25.10:
+            self.assertTrue(b'Error' in msg, reply)
         # VRFY addresses
         self._vrfy('address@example.com', valid=True)
         self._vrfy('does-not-exist', valid=False)
@@ -263,7 +272,7 @@ mech_list: %s %s
             self.s.login(auth_user, auth_pass)
         if use_tls:
             self.s.starttls()
-        failed = self.s.sendmail('root',[user_sent_to.login,'does-not-exist'],'''From: Rooty <root>
+        failed = self.s.sendmail('root', [user_sent_to.login, 'does-not-exist'], '''From: Rooty <root>
 To: "%s" <%s>
 Subject: This is test 1
 
@@ -271,9 +280,9 @@ Hello, nice to meet you.
 ''' % (user_sent_to.gecos, user_sent_to.login))
         #for addr in failed.keys():
         #    print '%s %d %s' % (addr, failed[addr][0], failed[addr][1])
-        self.assertEqual(len(failed),1,failed)
-        self.assertTrue('does-not-exist' in failed,failed)
-        self.assertEqual(failed['does-not-exist'][0],550,failed)
+        self.assertEqual(len(failed), 1, failed)
+        self.assertTrue('does-not-exist' in failed, failed)
+        self.assertEqual(failed['does-not-exist'][0], 550, failed)
 
         # Frighteningly, postfix seems to accept email before confirming
         # a successful write to disk for the recipient!
@@ -291,6 +300,7 @@ Hello, nice to meet you.
         time.sleep(1)
         with open(spool_file) as fh:
             contents = fh.read()
+        #print contents
         # Server-side added headers...
         self.assertTrue('\nReceived: ' in contents, contents)
         if use_tls and self.lsb_release['Release'] > 6.06:
@@ -339,7 +349,7 @@ Hello, nice to meet you.
 
     def _write_forward(self, user, contents):
         forward_filename = '/home/%s/.forward' % (user.login)
-        with open(forward_filename,'w') as fh:
+        with open(forward_filename, 'w') as fh:
             fh.write(contents)
         os.chown(forward_filename, user.uid, user.gid)
 
@@ -352,24 +362,22 @@ Hello, nice to meet you.
 
     def test_10_sending_mail_forward_xternal(self):
         '''Mail processed by commands in .forward'''
-    
+
         # Create user-writable redirected mbox destination
-        mbox, mbox_name = testlib.mkstemp_fill('',prefix='test-postfix.mbox-')
+        mbox, mbox_name = testlib.mkstemp_fill('', prefix='test-postfix.mbox-')
         mbox.close()
         os.chown(mbox_name, self.user.uid, self.user.gid)
-    
+
         # Create a script to run in the .forward
         redir, redir_name = testlib.mkstemp_fill('''#!/bin/bash
 /bin/cat > "%s"
-''' % (mbox_name),prefix='test-postfix.redir-')
+''' % (mbox_name), prefix='test-postfix.redir-')
         redir.close()
-        os.chmod(redir_name,0o755)
-    
-        self._write_forward(self.user,'|%s\n' % (redir_name))
+        os.chmod(redir_name, 0o755)
+
+        self._write_forward(self.user, '|%s\n' % (redir_name))
+        self._test_roundtrip_mail(self.user, spool_file=mbox_name)
 
-        # SKIP TESTING, FAILS IN TESTBED
-        #self._test_roundtrip_mail(self.user, spool_file=mbox_name)
-    
         os.unlink(redir_name)
         os.unlink(mbox_name)
 
@@ -378,29 +386,29 @@ Hello, nice to meet you.
 
         # First, create our "target" file
         secret = '/root/secret.txt'
-        with open(secret,'w') as fh:
+        with open(secret, 'w') as fh:
             fh.write('Secret information\n')
         os.chmod(secret, 0o700)
 
         # Now, create a symlink to the target (we're going to use /var/tmp
         # since we're assuming it, /root, /var/mail are on the same filesystem.
         # For most chroot testing, /tmp is mounted from the real machine.
-        if os.path.islink('/var/tmp/secret.link'):
+        if os.path.exists('/var/tmp/secret.link'):
             os.unlink('/var/tmp/secret.link')
-        self.assertEqual(subprocess.call(['su','-c','ln -s /root/secret.txt /var/tmp/secret.link',self.user.login]),0,"Symlink creation")
+        self.assertEqual(subprocess.call(['su', '-c', 'ln -s /root/secret.txt /var/tmp/secret.link', self.user.login]), 0, "Symlink creation")
 
         # Now, the hardlink, which in ubuntu's case needs to be done by root.
-        os.link('/var/tmp/secret.link','/var/mail/%s' % (self.user.login))
+        self.assertEqual(subprocess.call(['ln', '/var/tmp/secret.link', '/var/mail/%s' % (self.user.login)]), 0, "Symlink creation")
 
         # Email delivered to this user will be written to the root-owned
         # file now if the CVE is unfixed.
-        failed = self.s.sendmail('root',[self.user.login],'''From: Evil <root>
+        failed = self.s.sendmail('root', [self.user.login], '''From: Evil <root>
 To: "%s" <%s>
 Subject: This is an overwrite test
 
 Hello, nice to pwn you.
 ''' % (self.user.gecos, self.user.login))
-        self.assertEqual(len(failed),0,failed)
+        self.assertEqual(len(failed), 0, failed)
 
         # Pause for delivery
         time.sleep(2)
